//
// PhotoStamper | PhotoStamperApp.java
// Copyright (c) 2007-2008 Neil Johan
// http://www.neiljohan.com/software/photostamper/
//

import java.awt.Color;
import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.Graphics2D;
import java.awt.image.BufferedImage;
import java.io.File;
import java.io.FilenameFilter;
import java.io.IOException;
import java.util.Date;
import java.util.StringTokenizer;
import javax.imageio.*;

public class PhotoStamperApp implements FilenameFilter
{
	private static int sFontSize = 20;
	private static int sXOffset = 25;
	private static int sYOffset = 25;

	private static int sRed = 0;
	private static int sGreen = 0;
	private static int sBlue = 0;

	private static String sFont = "Arial";
	private static String sAnchor = "tl";
	private static String sExtension = "_stamped";

	public PhotoStamperApp(String[] aArgs)
	{
		try
		{
			if (aArgs.length == 0)
			{
				System.out.println("PhotoStamper 1.0. Copyright (c) 2007 Neil Broadbent.");
				System.out.println("http://www.neiljohan.com/software/photostamper/\n");
				System.out.println("Stamps the last modified date onto the image file.\n");
				System.out.println("PhotoStamper file [-s size] [-x xoffset] [-y yoffset] [-a anchor] [-c color]");
				System.out.println("	file	File or directory to be stampped.");
				System.out.println("	-s	Font size in pixels.");
				System.out.println("	-a	Anchor for text. One of tl, bl, tr, br. (top left, bottom left, etc).");
				System.out.println("	-x	x offset from the anchor.");
				System.out.println("	-y	y offset from the anchor.");
				System.out.println("	-c	color consisting of r:g:b values. 0-255.");
				System.out.println("e.g.");
				System.out.println("	PhotoStamper photo.jpg -s 50 -x 25 -y 25 -a tl -c 255:0:0");
				return;
			}

			Arguments args = Arguments.parse(aArgs);

			final String size = args.getFlagValue("s");
			if (size != null)
			{
				sFontSize = Integer.parseInt(size);
			}

			final String x = args.getFlagValue("x");
			if (x != null)
			{
				sXOffset = Integer.parseInt(x);
			}

			final String y = args.getFlagValue("y");
			if (y != null)
			{
				sYOffset = Integer.parseInt(y);
			}

			final String a = args.getFlagValue("a");
			if (a != null)
			{
				sAnchor = a;
			}

			final String c = args.getFlagValue("c");
			if (c != null)
			{
				StringTokenizer st = new StringTokenizer(c, ":");
				sRed = Integer.parseInt(st.nextToken());
				sGreen = Integer.parseInt(st.nextToken());
				sBlue = Integer.parseInt(st.nextToken());
			}

			String sourceLocation = aArgs[0];
			File source = new File(sourceLocation);
			if (source.isFile())
			{
				long lastModified = source.lastModified();
				File target = getTargetFile(source);
				stampFile(source, target, new Date(lastModified).toString());
			}
			else
			{
				timeStampDirectory(source);
			}
		}
		catch (Exception e)
		{
			e.printStackTrace();
		}
	}

	private static File getTargetFile(File aSourceFile) throws IOException
	{
		String sourceFileName = aSourceFile.getCanonicalPath();
		int separatorIndex = sourceFileName.lastIndexOf('.');
		String targetFileName = sourceFileName.substring(0, separatorIndex) +
			sExtension +
			sourceFileName.substring(separatorIndex);
		File target = new File(targetFileName);
		return target;
	}

	private static void stampFile(File aSourceFile, File aTargetFile, String aStamp) throws IOException
	{
		BufferedImage bi = ImageIO.read(aSourceFile);
		Graphics2D graphics = bi.createGraphics();
		Font font = new Font(sFont, Font.PLAIN, sFontSize);
		graphics.setFont(font);
		Color color = new Color(sRed, sGreen, sBlue);
		graphics.setColor(color);
		FontMetrics fm = graphics.getFontMetrics();
		final int stringWidth = fm.stringWidth(aStamp);
		int x = 0;
		int y = 0;
		if (sAnchor.equals("tl"))
		{
			x = sXOffset;
			y = sYOffset + sFontSize;
		}
		else if (sAnchor.equals("bl"))
		{
			x = sXOffset;
			y = bi.getHeight() - sYOffset;
		}
		else if (sAnchor.equals("tr"))
		{
			x = bi.getWidth() - stringWidth - sXOffset;
			y = sYOffset + sFontSize;
		}
		else if (sAnchor.equals("br"))
		{
			x = bi.getWidth() - stringWidth - sXOffset;
			y = bi.getHeight() - sYOffset;
		}
		else
		{
			Output.error("Invalid anchor: " + sAnchor + "\nValid anchors are tl, bl, tr, br");
		}
		graphics.drawString(aStamp, x, y);
		bi.flush();
		Output.debug("Writing " + aTargetFile.getCanonicalPath());
		ImageIO.write(bi, "jpg", aTargetFile);
	}

	private void timeStampDirectory(File aSourceDirectory) throws IOException
	{
		File[] files = aSourceDirectory.listFiles(this);
		final int numFile = files.length;
		for (int i=0; i<numFile; ++i)
		{
			long lastModified = files[i].lastModified();
			String dateStamp = new Date(lastModified).toString();
			stampFile(files[i], getTargetFile(files[i]), dateStamp);
		}
	}

	public boolean accept(File aDirectory, String aName)
	{
		int dotIndex = aName.lastIndexOf('.');
		if (dotIndex < 0 || dotIndex == aName.length())
		{
			return false;
		}
		if (getFileExtension(new File(aName)).equals("jpg"))
		{
			return true;
		}
		return false;
	}

	/**
	 * @param aFile
	 * @return extension of file - does not include the dot.
	 */
	private static String getFileExtension(File aFile)
	{
		try
		{
			String path = aFile.getCanonicalPath();
			int dotIndex = path.lastIndexOf('.');
			return path.substring(dotIndex + 1);
		}
		catch (IOException ioe)
		{
			return null;
		}
	}
}
